/*  _ __   __ _ ___ __  __
** | '_ \ / _` |_ _|  \/  | naim
** | | | | (_| || || |\/| | Copyright 1998-2000 Daniel Reed <n@ml.org>
** |_| |_|\__,_|___|_|  |_| ncurses-based, FAIM-based AIM client
** 
** FAIM Copyright 1998 Adam Fritzler <afritz@iname.com>
** FAIM uses code Copyright 1998 Brock Wilcox <awwaiid@iname.com>
*/
#include <naim.h>
#include <pwd.h>

extern WINDOW	*win_status;
extern buddywin_t	nullbwin, *curbwin;
extern faimconf_t	faimconf;
extern int	iamaway, gotdumped;
extern naimses_t	naimses;
extern time_t	now;

nFIRE_HANDLER(naim_newnick) {
	va_list	msg;
	char	*newnick;

	va_start(msg, sess);
	newnick = va_arg(msg, char *);
	va_end(msg);

	strcpy(naimses.sn, newnick);
	secs_setvar("SN", newnick);
}

nFIRE_HANDLER(naim_nickchange) {
	va_list	msg;
	char	*oldnick, *newnick;
	buddywin_t	*bwin;

	va_start(msg, sess);
	oldnick = va_arg(msg, char *);
	newnick = va_arg(msg, char *);
	va_end(msg);

	if ((bwin = bgetwin(oldnick)) != NULL) {
		strcpy(bwin->winname, newnick);
		bupdate();
	}
}

nFIRE_HANDLER(naim_buddylist) {
	va_list	msg;
	char	*screenname;
	int	online, away;

	va_start(msg, sess);
	screenname = va_arg(msg, char *);
	online = va_arg(msg, int);
	away = va_arg(msg, int);
	va_end(msg);

	raddbuddy(&faimconf, screenname, "Pre-existing buddy", NULL);
//	statprintf("!!!!! listbuddy { %s, %i, %i }\n", screenname, online, away);
}

nFIRE_HANDLER(naim_budprof) {
	buddylist_t	*buddy = faimconf.buddies;
	ignorelist_t	*idiot = faimconf.idiots;
	va_list	msg;
	char	*screenname;

	va_start(msg, sess);
	screenname = va_arg(msg, char *);
	va_end(msg);

	strncpy(faimconf.screenname, screenname, sizeof(faimconf.screenname)-1);
	faimconf.screenname[sizeof(faimconf.screenname)-1] = 0;

	sechof("Uploading buddy/block list and profile...\n");
	
	while (buddy != NULL) {
		firetalk_im_internal_add_buddy(sess, buddy->screenname);
		buddy = buddy->next;
	}

	while (idiot != NULL) {
		if (strcasecmp(idiot->notes, "block") == 0)
			firetalk_im_internal_add_deny(sess, idiot->screenname);
		idiot = idiot->next;
	}

	if (faimconf.profile != NULL)
		firetalk_set_info(sess, faimconf.profile);
	else
		firetalk_set_info(sess,
			"\"As a method of sending a missile to the higher, and even to the highest parts of the earth's atmospheric envelope, Professor Goddard's rocket is a practicable and therefore promising device.  It is when one considers the multiple-charge rocket as a traveler to the moon that one begins to doubt ... for after the rocket quits our air and really starts on its journey, its flight would be neither accelerated nor maintained by the explosion of the charges it then might have left. Professor Goddard, with his \"chair\" in Clark College and countenancing of the Smithsonian Institution, does not know the relation of action to re-action, and of the need to have something better than a vacuum against which to react ... Of course he only seems to lack the knowledge ladled out daily in high schools.\"<br>		-- New York Times Editorial, 1920<br>"
			"&nbsp;<br>"
			"naim-" NAIM_VERSION " / FireTalk-" LIBFIRETALK_VERSION "<br>"
			"Visit <a href=\"http://dev.n.ml.org/\">http://dev.n.ml.org/</a> for the latest naim."
		);

	if (iamaway)
		firetalk_set_away(sess, secs_getvar("awaymsg"));
}

nFIRE_HANDLER(naim_warned) {
	va_list	msg;
	char	*who;
	int	newlev;

	va_start(msg, sess);
	newlev = va_arg(msg, int);
	who = va_arg(msg, char *);
	va_end(msg);

	sechof("%s just warned you (%i).\n", who, newlev);
}

nFIRE_HANDLER(naim_buddy_idle) {
	va_list	msg;
	char	*who;
	long	idletime;

	va_start(msg, sess);
	who = va_arg(msg, char *);
	idletime = va_arg(msg, long);
	va_end(msg);

	if (idletime >= 10)
		bcoming(who, -1, 1);
	else
		bcoming(who, -1, 0);
}

nFIRE_HANDLER(naim_buddy_away) {
	va_list	msg;
	char	*who;

	va_start(msg, sess);
	who = va_arg(msg, char *);
	va_end(msg);

	bcoming(who, 1, -1);
}

nFIRE_HANDLER(naim_buddy_unaway) {
	va_list	msg;
	char	*who;

	va_start(msg, sess);
	who = va_arg(msg, char *);
	va_end(msg);

	bcoming(who, 0, -1);
}

nFIRE_HANDLER(naim_buddy_coming) {
	va_list	msg;
	char	*who;

	va_start(msg, sess);
	who = va_arg(msg, char *);
	va_end(msg);

	bcoming(who, 0, 0);
}

nFIRE_HANDLER(naim_buddy_going) {
	va_list	msg;
	char	*who;

	va_start(msg, sess);
	who = va_arg(msg, char *);
	va_end(msg);

	bgoing(who);
}

nFIRE_HANDLER(naim_act_handle) {
	va_list	msg;
	char	*who, *message,
		buf[1024];
	int	isautoreply;

	va_start(msg, sess);
	who = va_arg(msg, char *);
	isautoreply = va_arg(msg, int);
	message = va_arg(msg, char *);
	va_end(msg);

	snprintf(buf, sizeof(buf), "/me %s", message);
	naim_im_handle(sess, who, isautoreply, buf);
}

nFIRE_HANDLER(naim_im_handle) {
	va_list	msgg;
	buddywin_t	*bwin;
	struct tm	*tmptr = NULL;
	ignorelist_t	*idiot = NULL;
	char	*screenname, *msg = NULL;
	int	isautoreply = 0;

	tmptr = localtime(&now);
	assert(tmptr != NULL);
	idiot = faimconf.idiots;

	va_start(msgg, sess);
	screenname = va_arg(msgg, char *);
	isautoreply = va_arg(msgg, int);
	msg = va_arg(msgg, char *);
	va_end(msgg);

	if ((aimcmp(screenname, naimses.sn) == 0)
		&& (strncmp(msg, "<!--PING:", 9) == 0)) {
		char	buf[10];

		naimses.lag = now - atoi(msg+9);
		snprintf(buf, sizeof(buf), "%i", naimses.lag);
		secs_setvar("lag", buf);
		bupdate();
		debug_printf("naim_im_handle: resetting lag to %i\n",
			naimses.lag);
		return;
	}

	logim(screenname, naimses.sn, msg);

	while (idiot != NULL) {
		if (aimcmp(screenname, idiot->screenname) == 0) {
			if ((now - idiot->informed) > 1200) {
				char	igbuf[1200];

				snprintf(igbuf, sizeof(igbuf), "You are"
					" on my ignore list: <font"
					" size=+1 absz=14>%s"
					"</font><br>\n"
					"Your IM was not received by me,"
					" it was ignored. You will receive"
					" no more IM's from me (at least"
					" for 20 minutes).<br>\n<font"
					" size=-1 absz=10>This IM"
					" is automatic.</font>",
					idiot->notes);
				naim_send_im_away(screenname, igbuf);
				idiot->informed = now;
			}
			return;
		}
		idiot = idiot->next;
	}
	if (!iamaway && (secs_getvar_int("beeponim") == 1))
		beep();
	if ((secs_getvar_int("autobuddy") == 1)
		&& (bgetwin(screenname) == NULL)) {
		if (secs_getvar_int("syncbuddy") == 1)
			firetalk_im_add_buddy(sess, screenname);
		bnewwin(screenname);
	}
	if ((bwin = bgetwin(screenname)) == NULL) {
		nullbwin.waiting = 1;
		if (isautoreply == FALSE) {
			if (strncmp(msg, "/me ", 4) == 0)
				hwprintf(win_status, IMWIN_TEXT_R,
					"[%02i:%02i:%02i] *> <B>%s</B> %s<br>",
					tmptr->tm_hour, tmptr->tm_min,
					tmptr->tm_sec, screenname, msg+4);
			else
				hwprintf(win_status, IMWIN_TEXT_R,
					"[%02i:%02i:%02i] [<B>%s</B>] %s<br>",
					tmptr->tm_hour, tmptr->tm_min,
					tmptr->tm_sec, screenname, msg);
			if (iamaway && (*secs_getvar("awaymsg") != 0)
				&& ((now - nullbwin.informed) > 600)
				&& (aimcmp(screenname,
				   naimses.sn) != 0)) {
				sendaway(screenname);
				nullbwin.informed = now;
			}
		} else
			hwprintf(win_status, IMWIN_TEXT_R,
				"[%02i:%02i:%02i] [<B>%s</B> is away] %s<br>",
				tmptr->tm_hour, tmptr->tm_min,
				tmptr->tm_sec, screenname, msg);
		bupdate();
	} else {
		bwin->keepafterso = bwin->waiting = 1;
		if (isautoreply == FALSE) {
#if 0
			if (bwin->cryptkey != NULL) {
				char	*msgsave = msg;

				msg = calloc(2*strlen(msg)+1, 1);
				naim_unxor(msg, msgsave, bwin->cryptkey);
			}
#endif
			if (strncmp(msg, "/me ", 4) == 0)
				hwprintf(bwin->win, IMWIN_TEXT_R,
					"[%02i:%02i:%02i] * <B>%s</B> %s<br>", 
					tmptr->tm_hour, tmptr->tm_min,
					tmptr->tm_sec, screenname, msg+4);
			else
				hwprintf(bwin->win, IMWIN_TEXT_R,
					"[%02i:%02i:%02i] <B>%s</B>: %s<br>", 
					tmptr->tm_hour, tmptr->tm_min,
					tmptr->tm_sec, screenname, msg);
			if (iamaway && (*secs_getvar("awaymsg") != 0)
				&& ((now - bwin->informed) > 600)
				&& (aimcmp(screenname,
				   naimses.sn) != 0)) {
				sendaway(screenname);
				bwin->informed = now;
			}
			if (bwin->cryptkey != NULL)
				free(msg);
		} else
			hwprintf(bwin->win, IMWIN_TEXT_R,
				"[%02i:%02i:%02i] <B>%s</B> is away: %s<br>", 
				tmptr->tm_hour, tmptr->tm_min,
				tmptr->tm_sec, screenname, msg);
		bupdate();
	}
}

nFIRE_HANDLER(naim_error_msg) {
	va_list	msg;
	int	error;
	char	*target;
	buddywin_t	*bwin;

	va_start(msg, sess);
	error = va_arg(msg, int);
	target = va_arg(msg, char *);
	va_end(msg);

	if ((target != NULL) && ((bwin = bgetwin(target)) != NULL)) {
		struct tm	*tmptr;

		tmptr = localtime(&now);
		wprintf(bwin->win, IMWIN_STATUS, 1,
			"[%02i:%02i:%02i] ERROR: %s\n", tmptr->tm_hour,
			tmptr->tm_min, tmptr->tm_sec, firetalkerrors[error]);
		bwin->waiting = 1;
		bupdate();
	} else
		sechof("ERROR: %s: %s\n", target, firetalkerrors[error]);
};

nFIRE_HANDLER(naim_error_disconnect) {
	va_list	msg;
	int	error;

	va_start(msg, sess);
	error = va_arg(msg, int);
	va_end(msg);

	sechof("ERROR: Disconnect: %s\n", firetalkerrors[error]);
	gotdumped = 1;
}

nFIRE_HANDLER(naim_userinfo_handler) {
	va_list	msg;
	char	*SN, *info;
	int	warning, idle, class;

	va_start(msg, sess);
	SN = va_arg(msg, char *);
	info = va_arg(msg, char *);
	warning = va_arg(msg, int);
	idle = va_arg(msg, int);
	class = va_arg(msg, int);
	va_end(msg);

	hwprintf(curbwin->win, STAT_TEXT,
		"Information about %s:<br>", SN);
	hwprintf(curbwin->win, STAT_TEXT,
		"          <B>Class</B>:%s%s%s<br>",
			(class&FF_SUBSTANDARD)?" AIM":"",
			(class&FF_NORMAL)?" AOLamer":"",
			(class&FF_ADMIN)?" Admin":"");
	if (warning > 0)
	  hwprintf(curbwin->win, STAT_TEXT,
		"  <B>Warning level</B>: %i<br>", warning);
	if (idle > 0)
	  hwprintf(curbwin->win, STAT_TEXT,
		"      <B>Idle time</B>: <B>%i</B> minutes<br>",
		idle);
	if (info != NULL) {
	  hwprintf(curbwin->win, STAT_TEXT,
		"        <B>Profile</B>:<br>");
	  hwprintf(curbwin->win, STAT_TEXT, "%s<br><hr>", info);
	}
	bupdate();
}


buddywin_t      *cgetwin(char *roomname) {
	buddywin_t	*bwin;

	if ((bwin = bgetwin(roomname)) == NULL) {
		bnewwin(roomname);
		bwin = bgetwin(roomname);
		assert(bwin != NULL);
		bwin->keepafterso = bwin->ischat = 1;
		bwin->offline = 0;
		bupdate();
	} else {
		assert(bwin->ischat == 1);
		assert(bwin->keepafterso == 1);
//		assert(bwin->offline == 0);
		bwin->offline = 0;
	}
	return(bwin);
}

nFIRE_HANDLER(naim_chat_joined) {
	va_list	msg;
	buddywin_t	*bwin;
	char	*room;

	va_start(msg, sess);
	room = va_arg(msg, char *);
	va_end(msg);

	bwin = cgetwin(room);
	wechof(bwin, "Now talking in %s.\n", room);
	bupdate();
}

nFIRE_HANDLER(naim_chat_left) {
	va_list	msg;
//	buddywin_t	*bwin;
	char	*room;

	return;

	va_start(msg, sess);
	room = va_arg(msg, char *);
	va_end(msg);
}

nFIRE_HANDLER(naim_chat_kicked) {
	va_list	msg;
	buddywin_t	*bwin;
	char	*room, *by, *reason;

	va_start(msg, sess);
	room = va_arg(msg, char *);
	by = va_arg(msg, char *);
	reason = va_arg(msg, char *);
	va_end(msg);

	bwin = cgetwin(room);
	bwin->waiting = 1;
	wechof(bwin, "You have been removed from chat %s by %s (%s)\n",
			room, by, reason);
	bupdate();
}

nFIRE_HANDLER(naim_chat_act_handle) {
	va_list	msg;
	char	*room, *who, *message,
		buf[1024];
	int	isautoreply;

	va_start(msg, sess);
	room = va_arg(msg, char *);
	who = va_arg(msg, char *);
	isautoreply = va_arg(msg, int);
	message = va_arg(msg, char *);
	va_end(msg);

	snprintf(buf, sizeof(buf), "/me %s", message);
	naim_chat_getmessage(sess, room, who, isautoreply, buf);
}

nFIRE_HANDLER(naim_chat_getmessage) {
	va_list	msg;
	buddywin_t	*bwin;
	char	*room, *who, *message;
	int	isautoreply;
	struct tm	*tmptr;

	va_start(msg, sess);
	room = va_arg(msg, char *);
	who = va_arg(msg, char *);
	isautoreply = va_arg(msg, int);
	message = va_arg(msg, char *);
	va_end(msg);

	if (aimcmp(who, naimses.sn) == 0)
		return;

	tmptr = localtime(&now);

	bwin = cgetwin(room);
	bwin->waiting = 1;
	if (strncmp(message, "/me ", 4) == 0)
		hwprintf(bwin->win, IMWIN_TEXT_R,
			"[%02i:%02i:%02i] * <B>%s</B> %s<br>",
			tmptr->tm_hour, tmptr->tm_min,
			tmptr->tm_sec, who, message+4);
	else
		hwprintf(bwin->win, IMWIN_TEXT_R,
			"[%02i:%02i:%02i] &lt;<B>%s</B>&gt; %s<br>",
			tmptr->tm_hour, tmptr->tm_min,
			tmptr->tm_sec, who, message);
	bupdate();
}

nFIRE_HANDLER(naim_chat_invited) {
	va_list	msg;
	char	*room, *who, *message;

	va_start(msg, sess);
	room = va_arg(msg, char *);
	who = va_arg(msg, char *);
	message = va_arg(msg, char *);
	va_end(msg);

	sechof("%s invites you to chat %s: %s\n", who, room, message);
}

nFIRE_HANDLER(naim_chat_JOIN) {
	va_list	msg;
	buddywin_t	*bwin;
	char	*room, *who;

	va_start(msg, sess);
	room = va_arg(msg, char *);
	who = va_arg(msg, char *);
	va_end(msg);

	bwin = cgetwin(room);
	bwin->waiting = 1;
	wechof(bwin, "%s has joined chat %s\n", who, room);
	bupdate();
}

nFIRE_HANDLER(naim_chat_PART) {
	va_list	msg;
	buddywin_t	*bwin;
	char	*room, *who;

	va_start(msg, sess);
	room = va_arg(msg, char *);
	who = va_arg(msg, char *);
	va_end(msg);

	bwin = cgetwin(room);
	bwin->waiting = 1;
	wechof(bwin, "%s has left chat %s\n", who, room);
	bupdate();
}

nFIRE_HANDLER(naim_chat_KICK) {
	va_list	msg;
	buddywin_t	*bwin;
	char	*room, *who, *by, *reason;

	va_start(msg, sess);
	room = va_arg(msg, char *);
	who = va_arg(msg, char *);
	by = va_arg(msg, char *);
	reason = va_arg(msg, char *);
	va_end(msg);

	bwin = cgetwin(room);
	bwin->waiting = 1;
	wechof(bwin, "%s has been removed from chat %s by %s (%s)\n", who,
			room, by, reason);
	bupdate();
}

nFIRE_HANDLER(naim_chat_OP) {
	va_list	msg;
	buddywin_t	*bwin;
	char	*room, *who, *by;

	va_start(msg, sess);
	room = va_arg(msg, char *);
	who = va_arg(msg, char *);
	by = va_arg(msg, char *);
	va_end(msg);

	bwin = cgetwin(room);
	bwin->waiting = 1;
	wechof(bwin, "%s has been made an operator by %s\n", who, by);
	bupdate();
}

nFIRE_HANDLER(naim_chat_DEOP) {
	va_list	msg;
	buddywin_t	*bwin;
	char	*room, *who, *by;

	va_start(msg, sess);
	room = va_arg(msg, char *);
	who = va_arg(msg, char *);
	by = va_arg(msg, char *);
	va_end(msg);

	bwin = cgetwin(room);
	bwin->waiting = 1;
	wechof(bwin, "%s has been removed as an operator by %s\n", who, by);
	bupdate();
}

nFIRE_HANDLER(naim_chat_TOPIC) {
	va_list	msg;
	buddywin_t	*bwin;
	char	*room, *topic, *by;

	va_start(msg, sess);
	room = va_arg(msg, char *);
	topic = va_arg(msg, char *);
	by = va_arg(msg, char *);
	va_end(msg);

	bwin = cgetwin(room);
	bwin->waiting = 1;
	wechof(bwin, "%s has changed the topic on %s to %s\n", by, room,
		topic);
	bupdate();
}

time_t	lastctcp = 0;

nFIRE_CTCPHAND(naim_ctcp_VERSION) {
	if (lastctcp < now-1) {
		firetalk_subcode_send_reply(sess, from, "VERSION", "naim-" NAIM_VERSION " / FireTalk-" LIBFIRETALK_VERSION "; http://naim.n.ml.org/");
		lastctcp = now;
	}
}

nFIRE_CTCPHAND(naim_ctcp_PING) {
	if (lastctcp < now-1) {
		firetalk_subcode_send_reply(sess, from, "PING", args);
		lastctcp = now;
	}
}
